@@ -187,34 +187,88 @@ SRL_STATIC_INLINE srl_encoder_t *srl_dump_data_structure(pTHX_ srl_encoder_t *en
 #define DO_POK_REGEXP(enc, src, svt) /*no-op*/
 #endif
 
-#define _SRL_IF_SIMPLE_DIRECT_DUMP_SV(enc, src, svt)                    \
-    if (SvIOK(src)) {                                                   \
-    /* if its an integer its an integer */                              \
-        if (SvNOK(src) && SvPOK(src)) {                                 \
-            /* as far as I can tell the only strings which      */      \
-            /* set all three flags are engineering notation,    */      \
-            /* like "0E0" and friends - we especially need      */      \
-            /* to do this when the IV is 0, but we do it always */      \
-            /* if they put eng notation in, maybe then want it  */      \
-            /* out too. */                                              \
-            /* dump the string form */                                  \
-            srl_dump_svpv(aTHX_ enc, src);                              \
-        }                                                               \
-        else {                                                          \
-            /* dump ints */                                             \
-            srl_dump_ivuv(aTHX_ enc, src);                              \
-        }                                                               \
+#define _SRL_IF_SIMPLE_DIRECT_DUMP_SV(enc, src, svt)                        \
+    if (SvPOK(src)) {                                                       \
+        STRLEN L;                                                           \
+        char *PV= SvPV(src, L);                                             \
+        if ( SvIOK(src) ) {                                                 \
+            if ( SvIV(src) == 0 ) {                                         \
+                if ( L == 0 ) {                                             \
+                    /* zero length string, 0 int - same as PL_sv_no */      \
+                    srl_buf_cat_char(&enc->buf, SRL_HDR_FALSE);             \
+                }                                                           \
+                else if ( L == 1 && PV[0] == '0' ) {                        \
+                    /* its a true 0 */                                      \
+                    srl_buf_cat_char(&enc->buf, SRL_HDR_POS + 0);           \
+                }                                                           \
+                else {                                                      \
+                    /* must be a string */                                  \
+                    srl_dump_svpv(aTHX_ enc, src);                          \
+                }                                                           \
+            }                                                               \
+            else                                                            \
+            if (                                                            \
+                !L ||                                                       \
+                !isDIGIT(PV[L-1]) ||                                        \
+                (                                                           \
+                 SvIV(src) > 0                                              \
+                 ? ( PV[0] == '0' || !isDIGIT(PV[0]) )                      \
+                 : ( L < 2 || PV[0] != '-' || PV[1] == '0' || !isDIGIT(PV[1]) ) \
+                )                                                           \
+            ) {                                                             \
+                srl_dump_svpv(aTHX_ enc, src);                              \
+            }                                                               \
+            else {                                                          \
+                if ( SvNOK(src) ) {                                         \
+                    /* fallback to checking if the canonical stringified*/  \
+                    /* int is the same as the buffer */                     \
+                    sv_setiv(enc->scratch_sv,SvIV(src));                    \
+                    if ( sv_cmp(enc->scratch_sv,src) ) {                    \
+                        srl_dump_svpv(aTHX_ enc, src);                      \
+                    } else {                                                \
+                        srl_dump_ivuv(aTHX_ enc, src);                      \
+                    }                                                       \
+                } else {                                                    \
+                    srl_dump_ivuv(aTHX_ enc, src);                          \
+                }                                                           \
+            }                                                               \
+        }                                                                   \
+        else                                                                \
+        if ( SvNOK(src) ) {                                                 \
+            if ( L <= 8 ||                                                  \
+                !isDIGIT(PV[0]) ||                                          \
+                !isDIGIT(PV[L-1]) ||                                        \
+                PV[L-1] == '0' ||                                           \
+                 (                                                          \
+                  SvNV(src) > 0.0                                           \
+                  ? ( PV[0] == '.' || (PV[0] == '0' && PV[1] != '.') )      \
+                  : ( PV[0] != '-' || PV[1] == '.' || (PV[1] == '0' && PV[2] != '.')) \
+                )                                                           \
+            ) {                                                             \
+                srl_dump_svpv(aTHX_ enc, src);                              \
+            }                                                               \
+            else {                                                          \
+                srl_dump_nv(aTHX_ enc, src);                                \
+            }                                                               \
+        }                                                                   \
+        else {                                                              \
+            DO_POK_REGEXP(enc,src,svt)                                      \
+            srl_dump_svpv(aTHX_ enc, src);                                  \
+        }                                                                   \
+    }                                                                       \
+    else                                                                    \
+    if ( SvIOK(src) ) {                                                     \
+        srl_dump_ivuv(aTHX_ enc, src);                                  \
     }                                                                   \
     else                                                                \
     /* if its a float then its a float */                               \
     if (SvNOK(src)) {                                                   \
-        /* dump floats */                                               \
         srl_dump_nv(aTHX_ enc, src);                                    \
     }                                                                   \
     else                                                                \
     /* The POKp, IOKp, NOKp checks below deal with PVLV */              \
     /* if its POK or POKp, then we treat it as a string */              \
-    if (SvPOK(src) || SvPOKp(src)) {                                    \
+    if (SvPOKp(src)) {                                                  \
         DO_POK_REGEXP(enc,src,svt)                                      \
         srl_dump_svpv(aTHX_ enc, src);                                  \
     }                                                                   \
@@ -330,6 +384,7 @@ srl_destroy_encoder(pTHX_ srl_encoder_t *enc)
         SvREFCNT_dec(enc->string_deduper_hv);
 
     SvREFCNT_dec(enc->sereal_string_sv);
+    SvREFCNT_dec(enc->scratch_sv);
 
     Safefree(enc);
 }
@@ -367,6 +422,7 @@ srl_empty_encoder_struct(pTHX)
 
     enc->freezeobj_svhash = NULL;
     enc->sereal_string_sv = NULL;
+    enc->scratch_sv = NULL;
 
     return enc;
 }
@@ -389,6 +445,7 @@ srl_build_encoder_struct(pTHX_ HV *opt, sv_with_hash *options)
 
     enc = srl_empty_encoder_struct(aTHX);
     enc->flags = 0;
+    enc->scratch_sv= newSViv(0);
 
     /* load options */
     if (opt != NULL) {
@@ -569,7 +626,7 @@ srl_build_encoder_struct_alike(pTHX_ srl_encoder_t *proto)
         enc->sereal_string_sv = newSVpvs("Sereal");
     }
     enc->protocol_version = proto->protocol_version;
-
+    enc->scratch_sv= newSViv(0);
     DEBUG_ASSERT_BUF_SANE(&enc->buf);
     return enc;
 }
@@ -622,7 +679,7 @@ srl_write_header(pTHX_ srl_encoder_t *enc, SV *user_header_src, const U32 compre
      * + potentially uncompressed size varint
      * +  1 byte varint that indicates zero-length header */
     BUF_SIZE_ASSERT(&enc->buf, sizeof(SRL_MAGIC_STRING) + 1 + 1);
-    if (LIKELY( enc->protocol_version > 2 ))
+    if (expect_true( enc->protocol_version > 2 ))
       srl_buf_cat_str_s_nocheck(&enc->buf, SRL_MAGIC_STRING_HIGHBIT);
     else
       srl_buf_cat_str_s_nocheck(&enc->buf, SRL_MAGIC_STRING);
@@ -1044,6 +1101,13 @@ srl_dump_regexp(pTHX_ srl_encoder_t *enc, SV *sv)
     return;
 }
 
+#define ASSUME_BYTES_PER_TAG 4
+#define BUF_SIZE_ASSERT_AV(b,n) \
+        BUF_SIZE_ASSERT((b), 2 + SRL_MAX_VARINT_LENGTH + (1 * ASSUME_BYTES_PER_TAG * (n) ) )
+/* heuristic: 6 * n = liberal estimate of min size of n hashkeys */
+#define BUF_SIZE_ASSERT_HV(b, n) \
+        BUF_SIZE_ASSERT((b), 2 + SRL_MAX_VARINT_LENGTH + (2 * ASSUME_BYTES_PER_TAG * (n) ) )
+
 SRL_STATIC_INLINE void
 srl_dump_av(pTHX_ srl_encoder_t *enc, AV *src, U32 refcount)
 {
@@ -1053,7 +1117,7 @@ srl_dump_av(pTHX_ srl_encoder_t *enc, AV *src, U32 refcount)
     n = av_len(src)+1;
 
     /* heuristic: n is virtually the min. size of any element */
-    BUF_SIZE_ASSERT(&enc->buf, 2 + SRL_MAX_VARINT_LENGTH + n);
+    BUF_SIZE_ASSERT_AV(&enc->buf, n);
 
     if (n < 16 && refcount == 1 && !SRL_ENC_HAVE_OPTION(enc,SRL_F_CANONICAL_REFS)) {
         enc->buf.pos--; /* backup over previous REFN */
@@ -1112,6 +1176,7 @@ he_cmp_slow(const void *a, const void *b)
     return sv_cmp( HeSVKEY_force( *(HE **)b), HeSVKEY_force( *(HE **)a ) );
 }
 
+
 SRL_STATIC_INLINE void
 srl_dump_hv(pTHX_ srl_encoder_t *enc, HV *src, U32 refcount)
 {
@@ -1126,9 +1191,7 @@ srl_dump_hv(pTHX_ srl_encoder_t *enc, HV *src, U32 refcount)
         n = 0;
         while ((he = hv_iternext(src))) { ++n; }
 
-        /* heuristic: n     = ~min size of n values;
-         *            + 3*n = very conservative min size of n hashkeys if all COPY */
-        BUF_SIZE_ASSERT(&enc->buf, 2 + SRL_MAX_VARINT_LENGTH + 3 * n);
+        BUF_SIZE_ASSERT_HV(&enc->buf, n);
 
         if (n < 16 && refcount == 1 && !SRL_ENC_HAVE_OPTION(enc,SRL_F_CANONICAL_REFS)) {
             enc->buf.pos--; /* back up over the previous REFN */
@@ -1193,9 +1256,7 @@ srl_dump_hv(pTHX_ srl_encoder_t *enc, HV *src, U32 refcount)
         }
     } else {
         n= HvUSEDKEYS(src);
-        /* heuristic: n       = ~min size of n values;
-         *            + 3 * n = very conservative min size of n hashkeys if all COPY */
-        BUF_SIZE_ASSERT(&enc->buf, 2 + SRL_MAX_VARINT_LENGTH + 3 * n);
+        BUF_SIZE_ASSERT_HV(&enc->buf, n);
         if (n < 16 && refcount == 1 && !SRL_ENC_HAVE_OPTION(enc,SRL_F_CANONICAL_REFS)) {
             enc->buf.pos--; /* backup over the previous REFN */
             srl_buf_cat_char_nocheck(&enc->buf, SRL_HDR_HASHREF + n);
